/*********************************************************************
 *                                                                   *
 *                          RNAstrand.c                              *
 *                                                                   *
 *   Frontend of strand prediction of a structured RNA candidate.    *
 *                                                                   *
 *                c Kristin Reiche                                   *
 * $Id: RNAstrand.h,v 1.7 2007/03/06 13:24:27 kristin Exp $                                                            *
 *********************************************************************/

#ifndef RNASTRAND_H
#define RNASTRAND_H


#include <stdio.h>
#include <stdlib.h>

/* exit_with_help 
	exits the main programm and prints help message */
static void exit_with_help(void);

/* shows version of program and exits the program */
static void version(void);

/* reads CLUSTALW formatted file
	clust - pointer to filehandle, to be read
	alignedSeqs[] - array of strings where aligned sequences were stored 
	names - array of strings where names of the sequences were stored */
static int read_clustal(FILE *clust, char *alignedSeqs[], char *names[]);

/* reads MAF formatted file
        clust - pointer to filehandle, to be read
        alignedSeqs[] - array of strings where aligned sequences were stored 
        names - array of strings where names of the sequences were stored */
static int read_maf(FILE *clust, char *alignedSeqs[], char *names[]);

/* gets a slice of an alignment 
	sourceAln - array with sequences of source alignment
	destAln - pointer to array where slice will be stored
	from/ to - specifies slice, 1st column is 1 */
static void sliceAln(const char *sourceAln[], char *destAln[], int from, int to);

/* calculates mean pairwise identity of alignment 
	AS - array of aligned sequences
	returns mean pairw. identiy in percent */
static double meanPairID(const char *AS[]);

/* reverse complements of sequences in alignment
	AS - array of aligned sequences */
static void revAln(char *AS[]);

static void freeAln(char *AS[]);

/* prints the given values on stdout */
static void printVector(int trainType, double avgMFE, double meanMFE, double consMFE, double z_score, int n_seq, double id, double consFreqGU);
	
static void printDecVector(int dec, double prob, double decV, double SCI, double meanMFE, double consMFE, double z_score, 
		int ai, char * i, int f, int t, int l);
	
#endif
